/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef GWF_PLUGIN_H
#define GWF_PLUGIN_H

#ifdef __cplusplus
extern "C" {
#endif

#include <gtk/gtk.h>
#include <gmodule.h>
#include "gwf_xmlio.h"
#include "gwf_projdata.h"
#include "gwf_settings.h"
#include "gwf_projlist.h"
#include "gwf_source.h"
#include "gwf_treedata.h"
#include "gwf_error.h"

typedef struct  _GwfPlugin		GwfPlugin;
typedef struct	_GwfPluginList		GwfPluginList;
typedef struct	_GwfSrcPlugData		GwfSrcPlugData;

typedef GwfPlugin*	(*GwfPluginNewFunc)		(void);
typedef GwfProjData*	(*GwfPluginDataNewFunc)		(GwfProject *project);
typedef void		(*GwfPlugForeachFunc)		(GwfPlugin *plugin,gpointer user_data);

struct _GwfPlugin
{
   gchar			*name;
   gchar			*version;
   gchar			*href;
   GModule			*module;
   GPtrArray			*errmsg;

   GwfXmlLoadFunc		load_plugin;
   GwfXmlSaveFunc		save_plugin;

   GwfPluginDataNewFunc		new_data;

   GwfSettingsCreate		setting_create;

   GwfSrcCheckFunc		source_check;
   GList			*source_write;

   GwfTreeDataAttachFunc	tree_attach;
   GwfTreeDataFreeFunc		tree_free;
};

struct _GwfPluginList
{
   GHashTable		*datas;
};

struct _GwfSrcPlugData
{
   GwfWritePrior	prior;
   GwfSrcWrtFunc	func;
   GwfWriteFile		file;
};

GwfPluginList*	gwf_pluginlist_new		();
void		gwf_pluginlist_free		(GwfPluginList *list);
gint		gwf_pluginlist_add_plugin	(GwfPluginList *list,GwfPlugin *plugin);
void		gwf_pluginlist_del_plugin	(GwfPluginList *list,gchar *name);
GwfPlugin*	gwf_pluginlist_get_plugin	(GwfPluginList *list,gchar *name);

void		gwf_pluginlist_foreach_plugin	(GwfPluginList *list,GwfPlugForeachFunc func,gpointer user_data);

GwfPlugin*	gwf_plugin_load			(gchar *filename);
gint		gwf_plugin_unload		(GwfPlugin *plugin);

GwfPlugin*	gwf_plugin_new			(gchar *name);
void		gwf_plugin_free			(GwfPlugin *plugin);
void		gwf_plugin_add_srcwrt		(GwfPlugin *plugin,GwfWritePrior prior,GwfSrcWrtFunc func,GwfWriteFile file);

void		gwf_pluginlist_load_dir		(GwfPluginList *list,gchar *dir,GwfError *error);
void		gwf_pluginlist_reg_errors	(GwfPluginList *list,GwfError *error);
void		gwf_pluginlist_reg_settings	(GwfPluginList *list,GwfSettingsList *settings);
void		gwf_pluginlist_reg_xmlio	(GwfPluginList *list,GwfXmlIO *xml_io);
void		gwf_pluginlist_reg_treedatas	(GwfPluginList *list,GwfTreeDataList *treedatas);
void		gwf_pluginlist_reg_source	(GwfPluginList *list,GwfSource *srcwrt);
void		gwf_pluginlist_connect_projlist	(GwfPluginList *list,GwfProjlist *projlist);

void		gwf_plugin_reg_errors		(GwfPlugin *plugin,GwfError *error);
void		gwf_plugin_reg_settings		(GwfPlugin *plugin,GwfSettingsList *settings);
void		gwf_plugin_reg_xmlio		(GwfPlugin *plugin, GwfXmlIO *xml_io);
void		gwf_plugin_reg_treedata		(GwfPlugin *plugin,GwfTreeDataList *treedatas);
void		gwf_plugin_reg_source		(GwfPlugin *plugin,GwfSource *srcwrt);

#ifdef __cplusplus
}
#endif

#endif
