/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gwf_tools.h"
#include <gtk/gtk.h>
#include <string.h>
#include "gwf_projlist.h"
#include "gwf_project.h"

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_DIRENT_H
#  include <dirent.h>
#endif

typedef struct _GtkTypeConv	GtkTypeConv;

struct _GtkTypeConv
{
  gchar		*str;
  GtkType	type;
};

static void	parent_destroy_cb 	(GtkObject *object,gpointer user_data);
static void	child_destroy_cb 	(GtkObject *object,gpointer user_data);
static gboolean	window_delete_event_hide(GtkWidget *widget,GdkEvent *event,gpointer user_data);

static GtkTypeConv		type_conv[] =	{
						  {"void",GTK_TYPE_NONE},
						  {"gchar",GTK_TYPE_CHAR},
						  {"guchar",GTK_TYPE_UCHAR},
						  {"gboolean",GTK_TYPE_BOOL},
						  {"gint",GTK_TYPE_INT},
						  {"guint",GTK_TYPE_UINT},
						  {"gshort",GTK_TYPE_INT},
						  {"gushort",GTK_TYPE_INT},
						  {"glong",GTK_TYPE_LONG},
						  {"gulong",GTK_TYPE_ULONG},
						  {"gfloat",GTK_TYPE_FLOAT},
						  {"gdouble",GTK_TYPE_DOUBLE},
						  {"gchar*",GTK_TYPE_STRING},
						  {"gpointer",GTK_TYPE_POINTER},
						  {"gconstpointer",GTK_TYPE_POINTER}
						};
static gchar		*types[] = {
					"GTK_TYPE_INVALID",
					"GTK_TYPE_NONE",
					"GTK_TYPE_CHAR",
					"GTK_TYPE_UCHAR",
					"GTK_TYPE_BOOL",
					"GTK_TYPE_INT",
					"GTK_TYPE_UINT",
					"GTK_TYPE_LONG",
					"GTK_TYPE_ULONG",
					"GTK_TYPE_FLOAT",
					"GTK_TYPE_DOUBLE",
					"GTK_TYPE_STRING",
					"GTK_TYPE_ENUM",
					"GTK_TYPE_FLAGS",
					"GTK_TYPE_BOXED",
					"GTK_TYPE_POINTER",
					"GTK_TYPE_SIGNAL",
					"GTK_TYPE_ARGS",
					"GTK_TYPE_CALLBACK",
					"GTK_TYPE_C_CALLBACK",
					"GTK_TYPE_FOREIGN",
					"GTK_TYPE_OBJECT"				
				   }; 	

gint
gtk_clist_get_row_by_text (GtkCList *clist,gint column,gchar *text)
{
  guint 	row;
  gchar		*temp = NULL;
  
  g_return_val_if_fail(clist != NULL,-1);
  g_return_val_if_fail(text != NULL,-1);
  
  for ( row = 0 ; row < clist->rows ; row++)
  {
    gtk_clist_get_text(clist,row,column,&temp);
    if (strcmp(temp,text) == 0)
      return row;
  }
  return -1;
}

gchar*
gtk_clist_get_sel_row_col_text (GtkCList *clist,gint column)
{
  gint 		row;
  gchar 	*txt;
  
  g_return_val_if_fail(clist != NULL,NULL);
  
  if (!clist->selection)
    return NULL;

  row = GPOINTER_TO_INT(clist->selection->data);
  
  if (!gtk_clist_get_text(clist,row,column,&txt))
    return NULL;

  return g_strdup(txt);
}

void
gwf_window_show	(GtkWidget *window)
{
  g_return_if_fail(window != NULL);
  g_return_if_fail(GTK_IS_WINDOW(window));
  
  gtk_widget_show(window);
  gdk_window_show(window->window);
  gdk_window_raise(window->window);
}

void
gwf_window_set_title(GtkWindow *window,gchar *windowname,gchar *project)
{
  gchar		*temp;

  temp = g_strdup_printf("%s : %s",windowname,NULL_TO_EMPTY(project));
  gtk_window_set_title(window,temp);
  g_free(temp);
}

void
gwf_window_set_title_current(GtkWindow *window,gchar *windowname,GwfProjlist *projlist)
{
  GwfProject	*project;
  gchar		*projname;

  project = gwf_projlist_get_current(projlist);

  projname = (project) ? gwf_project_get_widname(project) : NULL;
  gwf_window_set_title(window,windowname,projname);
}

void
gwf_parent_child_windows_connect(GtkWindow *parent,GtkWindow *child)
{
  gtk_signal_connect(GTK_OBJECT(parent),"destroy",
                     (GtkSignalFunc)parent_destroy_cb,
                     (gpointer)child);
  gtk_signal_connect(GTK_OBJECT(child),"destroy",
                     (GtkSignalFunc)child_destroy_cb,
                     (gpointer)parent);
}

static void
parent_destroy_cb (GtkObject *object,gpointer user_data)
{
  gtk_widget_destroy(GTK_WIDGET(user_data));
}

static void
child_destroy_cb (GtkObject *object,gpointer user_data)
{
  gtk_signal_disconnect_by_data(GTK_OBJECT(user_data),(gpointer)object);
}

void
gwf_window_hide_on_destroy (GtkWindow *window)
{
  gtk_signal_connect(GTK_OBJECT(window),"delete_event",
                     GTK_SIGNAL_FUNC(window_delete_event_hide),
                     NULL);
}

static gboolean
window_delete_event_hide (GtkWidget *widget,GdkEvent *event,gpointer user_data)
{
  gtk_widget_hide(widget);
  return TRUE;
}



GtkCTreeNode*
gtk_ctree_add_node(GtkCTree *tree,GtkCTreeNode *prnt,gchar *name)
{
  gchar		*text[1];
  
  g_return_val_if_fail(tree != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);

  text[0] = name;
  return gtk_ctree_insert_node(tree,prnt,NULL,text,0,NULL,NULL,NULL,NULL,FALSE,FALSE);
}

void
gtk_ctree_alter_node(GtkCTree *tree,GtkCTreeNode *node,gchar *text)
{
  guint8	spacing;
  GdkPixmap	*pixmap_closed;
  GdkBitmap	*mask_closed;
  GdkPixmap	*pixmap_open;
  GdkBitmap	*mask_open;
  gboolean	is_leaf;
  gboolean	is_expanded;

  g_return_if_fail(tree != NULL);
  g_return_if_fail(node != NULL);
  g_return_if_fail(text != NULL);
  
  if (!gtk_ctree_get_node_info(tree,node,NULL,&spacing,&pixmap_closed,&mask_closed,
      			       &pixmap_open,&mask_open,&is_leaf,&is_expanded))
    return;
  
  gtk_ctree_set_node_info(tree,node,text,spacing,pixmap_closed,mask_closed,
                          pixmap_open,mask_open,is_leaf,is_expanded);
}

void
gtk_ctree_freeze (GtkCTree *tree)
{
  g_return_if_fail(tree != NULL);
  gtk_clist_freeze(GTK_CLIST(tree));
}

void
gtk_ctree_thaw (GtkCTree *tree)
{
  g_return_if_fail(tree != NULL);
  gtk_clist_thaw(GTK_CLIST(tree));
}

GtkCTreeNode*
gtk_ctree_find_item(GtkCTree *tree,GtkCTreeNode *parent,gchar *item)
{
  GtkCTreeNode 	*tmp;
  gchar		*text;
  
  g_return_val_if_fail(tree != NULL,NULL);
  g_return_val_if_fail(parent != NULL,NULL);
  g_return_val_if_fail(item != NULL,NULL);

  tmp = GTK_CTREE_ROW(parent)->children;
  
  while (tmp)
  {
    if (!gtk_ctree_get_node_info(tree,tmp,&text,NULL,NULL,NULL,NULL,NULL,NULL,NULL))
      return NULL;
    if (strcmp(item,text) == 0)
      return tmp;
      
    tmp = GTK_CTREE_ROW(tmp)->sibling;
  }
  
  return NULL;
}

GtkCTreeNode*
gtk_ctree_find_item_part(GtkCTree *tree,GtkCTreeNode *parent,gchar *item)
{
  GtkCTreeNode 	*tmp;
  gchar		*text;
  
  g_return_val_if_fail(tree != NULL,NULL);
  g_return_val_if_fail(parent != NULL,NULL);
  g_return_val_if_fail(item != NULL,NULL);

  tmp = GTK_CTREE_ROW(parent)->children;
  
  while (tmp)
  {
    if (!gtk_ctree_get_node_info(tree,tmp,&text,NULL,NULL,NULL,NULL,NULL,NULL,NULL))
      return NULL;
      
    if (strncmp(item,text,strlen(item)) == 0)
      return tmp;
      
    tmp = GTK_CTREE_ROW(tmp)->sibling;
  }
  
  return NULL;
}

GtkCTreeNode*
gtk_ctree_find_project(GtkCTree *tree,GwfProject *project)
{
  g_return_val_if_fail(tree != NULL,NULL);
  g_return_val_if_fail(project != NULL,NULL);
  
  return gtk_ctree_find_by_row_data(tree,NULL,(gpointer)project);
}

GtkCTreeNode*
gtk_ctree_find_project_item(GtkCTree *tree,GwfProject *project,gchar *item)
{
  GtkCTreeNode 	*tmp;
  
  g_return_val_if_fail(tree != NULL,NULL);
  g_return_val_if_fail(project != NULL,NULL);
  g_return_val_if_fail(item != NULL,NULL);
  
  tmp = gtk_ctree_find_project(tree,project);
  if (!tmp)
    return NULL;
    
  return gtk_ctree_find_item(tree,tmp,item);  
}



gint
gtk_ctree_insert_text_value(GtkCTree *tree,GtkCTreeNode *parent,gchar *item,gchar *value)
{
  gchar		*tmp = NULL;

  g_return_val_if_fail(tree != NULL,0);
  g_return_val_if_fail(parent != NULL,0);
  g_return_val_if_fail(item != NULL,0);

  tmp = g_strdup_printf("%s : %s",item,NULL_TO_EMPTY(value));
  if (!gtk_ctree_add_node(tree,parent,tmp))
  {
    gtk_ctree_remove_node(tree,parent);
    g_free(tmp);
    return 0;
  }
  g_free(tmp);
  return 1;
}

gint
gtk_ctree_insert_bool_value(GtkCTree *tree,GtkCTreeNode *parent,gchar *item,gboolean value)
{
  g_return_val_if_fail(tree != NULL,0);
  g_return_val_if_fail(parent != NULL,0);
  g_return_val_if_fail(item != NULL,0);

  return gtk_ctree_insert_text_value(tree,parent,item,BOOL_TO_STR(value));
}

gint
gtk_ctree_insert_float_value(GtkCTree *tree,GtkCTreeNode *parent,gchar *item,gfloat value)
{
  gchar		*tmp;
  gint		result;

  g_return_val_if_fail(tree != NULL,0);
  g_return_val_if_fail(parent != NULL,0);
  g_return_val_if_fail(item != NULL,0);

  tmp = g_strdup_printf("%5.2f",value);
  result = gtk_ctree_insert_text_value(tree,parent,item,tmp);
  g_free(tmp);

  return result;
}

gint
gtk_ctree_insert_int_value(GtkCTree *tree,GtkCTreeNode *parent,gchar *item,glong value)
{
  gchar		*tmp;
  gint		result;

  g_return_val_if_fail(tree != NULL,0);
  g_return_val_if_fail(parent != NULL,0);
  g_return_val_if_fail(item != NULL,0);

  tmp = g_strdup_printf("%ld",value);
  result = gtk_ctree_insert_text_value(tree,parent,item,tmp);
  g_free(tmp);

  return result;
}

void
gtk_ctree_alter_text_value(GtkCTree *tree,GtkCTreeNode *node,gchar *item,gchar *value)
{
  gchar		*tmp = NULL;

  g_return_if_fail(tree != NULL);
  g_return_if_fail(node != NULL);
  g_return_if_fail(item != NULL);

  tmp = g_strdup_printf("%s : %s",item,NULL_TO_EMPTY(value));
  gtk_ctree_alter_node(tree,node,tmp);
  g_free(tmp);
}

void
gtk_ctree_alter_bool_value(GtkCTree *tree,GtkCTreeNode *node,gchar *item,gboolean value)
{
  g_return_if_fail(tree != NULL);
  g_return_if_fail(node != NULL);
  g_return_if_fail(item != NULL);

  gtk_ctree_alter_text_value(tree,node,item,BOOL_TO_STR(value));
}

void
gtk_ctree_alter_float_value(GtkCTree *tree,GtkCTreeNode *node,gchar *item,gfloat value)
{
  gchar		*tmp;

  g_return_if_fail(tree != NULL);
  g_return_if_fail(node != NULL);
  g_return_if_fail(item != NULL);

  tmp = g_strdup_printf("%5.2f",value);
  gtk_ctree_alter_text_value(tree,node,item,tmp);
  g_free(tmp);
}

void
gtk_ctree_alter_int_value(GtkCTree *tree,GtkCTreeNode *node,gchar *item,glong value)
{
  gchar		*tmp;

  g_return_if_fail(tree != NULL);
  g_return_if_fail(node != NULL);
  g_return_if_fail(item != NULL);

  tmp = g_strdup_printf("%ld",value);
  gtk_ctree_alter_text_value(tree,node,item,tmp);
  g_free(tmp);
}

void
gtk_combo_entry_add_item (GtkEntry *entry,gchar *text)
{
  GtkCombo	*combo;
  GtkWidget	*item;
  GtkWidget	*label;

  combo = GTK_COMBO(GTK_WIDGET(entry)->parent);
  item = gtk_list_item_new();
  gtk_widget_show(item);
  label = gtk_label_new(text);
  gtk_widget_show(label);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_container_add(GTK_CONTAINER(item),label);
  gtk_combo_set_item_string(combo,GTK_ITEM(item),text);
  gtk_container_add(GTK_CONTAINER(combo->list),item);
}

gchar*
gtk_type_to_string (GtkType type)
{
  return ((type > 0) && (type <= GTK_TYPE_FUNDAMENTAL_LAST)) ? types[type] : "GTK_TYPE_INVALID";
}

GtkType
string_to_gtk_type (gchar *type)
{
  gint		i;

  if (!type)
    return GTK_TYPE_INVALID;

  for ( i = 0; i <= GTK_TYPE_FUNDAMENTAL_LAST ; i++)
    if (strcmp(types[i],type) == 0) return i;

  return GTK_TYPE_INVALID;
}

GtkType
find_best_gtk_type(gchar *gtktype,gchar *strtype)
{
  GtkType	type;
  gchar		*tmp;
  gint		i;

  g_return_val_if_fail(gtktype != NULL,GTK_TYPE_INVALID);
  g_return_val_if_fail(strtype != NULL,GTK_TYPE_INVALID);

  type = string_to_gtk_type(gtktype);
  if (type != GTK_TYPE_INVALID)
    return type;

  tmp = g_strdup(strtype);
  g_strdown(tmp);

  for ( i = 0; i < 15 ; i++)
    if (strcmp(tmp,type_conv[i].str) == 0)
      type = type_conv[i].type;

  return type;
}

GList*
gwf_open_dir (gchar *dir)
{
  gint		n,i;
  DIR*		dir_ptr;
  struct dirent	*entry;
  GList		*fnames = NULL;

  g_return_val_if_fail (dir != NULL,NULL);

  dir_ptr = opendir(dir);
  if (!dir_ptr)
    return NULL;

  while ((entry = readdir(dir_ptr)) != NULL)
  {
    entry = readdir(dir_ptr);
    if (entry)
      fnames = g_list_append(fnames,g_strdup(entry->d_name));
  }

  closedir(dir_ptr);

  return fnames;
}